@@ -18,6 +18,7 @@
  */
 
 #include "config.h"
+#include "ass_compat.h"
 
 #include <inttypes.h>
 #include <ft2build.h>
@@ -43,8 +44,6 @@ static unsigned font_hash(void *buf, size_t len)
     hval = fnv_32a_str(desc->family, FNV1_32A_INIT);
     hval = fnv_32a_buf(&desc->bold, sizeof(desc->bold), hval);
     hval = fnv_32a_buf(&desc->italic, sizeof(desc->italic), hval);
-    hval = fnv_32a_buf(&desc->treat_family_as_pattern,
-            sizeof(desc->treat_family_as_pattern), hval);
     hval = fnv_32a_buf(&desc->vertical, sizeof(desc->vertical), hval);
     return hval;
 }
@@ -59,8 +58,6 @@ static unsigned font_compare(void *key1, void *key2, size_t key_size)
         return 0;
     if (a->italic != b->italic)
         return 0;
-    if (a->treat_family_as_pattern != b->treat_family_as_pattern)
-        return 0;
     if (a->vertical != b->vertical)
         return 0;
     return 1;
@@ -81,8 +78,6 @@ static void bitmap_destruct(void *key, void *value)
         ass_free_bitmap(v->bm);
     if (v->bm_o)
         ass_free_bitmap(v->bm_o);
-    if (v->bm_s)
-        ass_free_bitmap(v->bm_s);
     if (k->type == BITMAP_CLIP)
         free(k->u.clip.text);
     free(key);
@@ -92,11 +87,12 @@ static void bitmap_destruct(void *key, void *value)
 static size_t bitmap_size(void *value, size_t value_size)
 {
     BitmapHashValue *val = value;
+    size_t size = sizeof(BitmapHashKey) + sizeof(BitmapHashValue);
+    if (val->bm)
+        size += sizeof(Bitmap) + val->bm->stride * val->bm->h;
     if (val->bm_o)
-        return val->bm_o->w * val->bm_o->h * 3;
-    else if (val->bm)
-        return val->bm->w * val->bm->h * 3;
-    return 0;
+        size += sizeof(Bitmap) + val->bm_o->stride * val->bm_o->h;
+    return size;
 }
 
 static unsigned bitmap_hash(void *key, size_t key_size)
@@ -132,7 +128,7 @@ static void composite_destruct(void *key, void *value)
         ass_free_bitmap(v->bm_o);
     if (v->bm_s)
         ass_free_bitmap(v->bm_s);
-    free(k->str.data);
+    free(k->bitmaps);
     free(key);
     free(value);
 }
@@ -140,11 +136,41 @@ static void composite_destruct(void *key, void *value)
 static size_t composite_size(void *value, size_t value_size)
 {
     CompositeHashValue *val = value;
+    size_t size = sizeof(CompositeHashKey) + sizeof(CompositeHashValue);
+    if (val->bm)
+        size += sizeof(Bitmap) + val->bm->stride * val->bm->h;
     if (val->bm_o)
-        return val->bm_o->w * val->bm_o->h * 3;
-    else if (val->bm)
-        return val->bm->w * val->bm->h * 3;
-    return 0;
+        size += sizeof(Bitmap) + val->bm_o->stride * val->bm_o->h;
+    if (val->bm_s)
+        size += sizeof(Bitmap) + val->bm_s->stride * val->bm_s->h;
+    return size;
+}
+
+static unsigned composite_hash(void *key, size_t key_size)
+{
+    CompositeHashKey *k = key;
+    unsigned hval = filter_hash(&k->filter, key_size);
+    for (size_t i = 0; i < k->bitmap_count; ++i) {
+        hval = fnv_32a_buf(&k->bitmaps[i].image, sizeof(k->bitmaps[i].image), hval);
+        hval = fnv_32a_buf(&k->bitmaps[i].x, sizeof(k->bitmaps[i].x), hval);
+        hval = fnv_32a_buf(&k->bitmaps[i].y, sizeof(k->bitmaps[i].y), hval);
+    }
+    return hval;
+}
+
+static unsigned composite_compare(void *a, void *b, size_t key_size)
+{
+    CompositeHashKey *ak = a;
+    CompositeHashKey *bk = b;
+    if (ak->bitmap_count != bk->bitmap_count)
+        return 0;
+    for (size_t i = 0; i < ak->bitmap_count; ++i) {
+        if (ak->bitmaps[i].image != bk->bitmaps[i].image ||
+            ak->bitmaps[i].x != bk->bitmaps[i].x ||
+            ak->bitmaps[i].y != bk->bitmaps[i].y)
+            return 0;
+    }
+    return filter_compare(&ak->filter, &bk->filter, key_size);
 }
 
 // outline cache
@@ -175,10 +201,10 @@ static void outline_destruct(void *key, void *value)
 {
     OutlineHashValue *v = value;
     OutlineHashKey *k = key;
-    if (v->outline)
-        outline_free(v->lib, v->outline);
-    if (v->border)
-        outline_free(v->lib, v->border);
+    outline_free(v->outline);
+    free(v->outline);
+    outline_free(v->border);
+    free(v->border);
     if (k->type == OUTLINE_DRAWING)
         free(k->u.drawing.text);
     free(key);
@@ -237,6 +263,8 @@ Cache *ass_cache_create(HashFunction hash_func, HashCompare compare_func,
                         size_t key_size, size_t value_size)
 {
     Cache *cache = calloc(1, sizeof(*cache));
+    if (!cache)
+        return NULL;
     cache->buckets = 0xFFFF;
     cache->hash_func = hash_simple;
     cache->compare_func = compare_simple;
@@ -251,6 +279,10 @@ Cache *ass_cache_create(HashFunction hash_func, HashCompare compare_func,
     cache->key_size = key_size;
     cache->value_size = value_size;
     cache->map = calloc(cache->buckets, sizeof(CacheItem *));
+    if (!cache->map) {
+        free(cache);
+        return NULL;
+    }
 
     return cache;
 }
@@ -261,8 +293,16 @@ void *ass_cache_put(Cache *cache, void *key, void *value)
     CacheItem **bucketptr = &cache->map[bucket];
 
     CacheItem *item = calloc(1, sizeof(CacheItem));
+    if (!item)
+        return NULL;
     item->key = malloc(cache->key_size);
     item->value = malloc(cache->value_size);
+    if (!item->key || !item->value) {
+        free(item->key);
+        free(item->value);
+        free(item);
+        return NULL;
+    }
     memcpy(item->key, key, cache->key_size);
     memcpy(item->value, value, cache->value_size);
 
